@@ -10,10 +10,12 @@ sub _build_limit {
 }
 
 sub _build_sql_select {
-    my($class, $me, $h) = @_;
-    my $sql = $class->SUPER::_build_sql_select($me, $h);
-    return $sql unless defined $h->{LimitOffset};
-    return 'SELECT * FROM ('.$sql.') WHERE ROWNUM <= '.$h->{LimitOffset}[0];
+    my($class, $me) = @_;
+    my $sql = $class->SUPER::_build_sql_select($me);
+    return $sql unless defined (my $limoff = $me->{build_data}{LimitOffset});
+    return 'SELECT * FROM ('.$sql.') WHERE ROWNUM <= '.$limoff->[0] unless $limoff->[1];
+    # If we have an offset then we must add the "_DBO_ROWNUM_" column to the result set
+    return 'SELECT * FROM (SELECT A.*, ROWNUM AS "_DBO_ROWNUM_" FROM ('.$sql.') A WHERE ROWNUM <= '.($limoff->[0] + $limoff->[1]).') WHERE "_DBO_ROWNUM_" > '.$limoff->[1];
 }
 
 sub _alias_preference {
